---
title: 'hIPPYlib: An Extensible Software Framework for Large-Scale Inverse Problems'
tags:
  - Python
  - Infinite-dimensional inverse problems
  - adjoint-based methods
  - numerical optimization
  - Bayesian inference
  - uncertainty quantification
  - PDE toolkit
authors:
  - name: Umberto Villa
    affiliation: 1
  - name: Noemi Petra
    affiliation: 2
  - name: Omar Ghattas
    affiliation: 3
affiliations:
  - name: Institute for Computational Engineering & Sciences, The University of Texas at Austin
    index: 1
  - name: Applied Mathematics, School of Natural Sciences, University of California, Merced
    index: 2
  - name: Institute for Computational Engineering & Sciences, Department of Mechanical Engineering, and Department of Geological Sciences, The University of Texas at Austin
    index: 3
date: 1 June 2018
bibliography: paper.bib
---

# Summary

[`hIPPYlib`](https://hippylib.github.io) [@VillaPetraGhattas18] implements
state-of-the-art scalable algorithms for deterministic and Bayesian inverse problems 
governed by partial differential equations (PDEs); see, e.g., [@Bui-ThanhGhattasMartinEtAl13], [@PetraMartinStadlerEtAl14] for an introduction to infinite
dimensional PDE-constrained Bayesian inverse problems.
`hIPPYlib` builds on the parallel finite element library [`FEniCS`](http://fenicsproject.org/), [@LoggWells10], [@LoggMardalGarth12], [@LangtangenLogg17],
for the discretization of the PDE and on `PETSc`, [@petsc-web-page]
for scalable and efficient linear algebra operations and solvers.

Conceptually, `hIPPYlib` can be viewed as a toolbox that provides the
building blocks for experimenting new ideas and developing scalable
algorithms for PDE-constrained deterministic and Bayesian inverse problems.

In `hIPPYlib` the user can express the forward PDE and the likelihood in
weak form using the friendly, compact, near-mathematical notation of
`FEniCS`, which will then automatically generate efficient code for the
discretization.  Linear and nonlinear, and stationary and
time-dependent PDEs are supported in `hIPPYlib`.
For stationary problems, gradient and Hessian information can be
automatically generated by `hIPPYlib` using `FEniCS` symbolic differentiation
of the relevant weak forms. For time-dependent problems, instead, symbolic
differentiation can only be used for the spatial terms, and the contribution
to gradients and Hessians arising from the time dynamics needs to be provided
by the user.

Noise and prior covariance operators are modeled as inverses of
elliptic differential operators allowing us to build on existing fast
multigrid solvers for elliptic operators without explicitly
constructing the dense covariance operator.

The key property of the algorithms underlying `hIPPYlib` is that solution
of the deterministic and Bayesian inverse problem is computed
at a cost, measured in forward PDE solves, that is independent of the
parameter dimension.

`hIPPYlib` provides a robust implementation of the inexact
Newton-conjugate gradient algorithm to compute the maximum a posterior
(MAP) point. The gradient and Hessian actions are
computed via their weak form specification in `FEniCS` by
constraining the state and adjoint variables to satisfy the forward
and adjoint problem. The Newton system is solved inexactly by early
termination of CG iterations via Eisenstat-Walker (to prevent
oversolving) and Steihaug (to avoid negative curvature)
criteria. Two globalization techniques are available to the user:
Armijo back-tracking line search and trust region.

In `hIPPYlib`, the posterior covariance is approximated by the
inverse of the Hessian of the negative log posterior evaluated at
the MAP point. This Gaussian approximation is exact when the
parameter-to-observable map is linear; otherwise, its logarithm agrees
to two derivatives with the log posterior at the MAP point, and thus it
can serve as a proposal for Hessian-based Markov chain Monte Carlo (MCMC)
methods. `hIPPYlib` makes the construction of the posterior covariance
tractable by invoking a low-rank approximation of the Hessian of the
log likelihood.

`hIPPYlib` also offers scalable methods for sample generation.
To sample large scale spatially correlated Gaussian random fields from the prior
distribution, `hIPPYlib`  implements a new method that strongly relies on the
structure of the covariance operator defined as the inverse of a differential operator:
by exploiting the assembly procedure of finite element matrices `hIPPYlib` constructs a sparse Cholesky-like rectangular decomposition of the precision operator.
To sample from a local Gaussian approximation to the posterior (such as at the MAP point)
`hIPPYlib` exploits the low rank factorization of the Hessian of the
log likelihood to correct samples from the prior distribution.
Finally, to explore the posterior distribution, `hIPPYlib` implements
dimension independent MCMC sampling methods enchanted by Hessian information.

Finally, randomized and probing algorithms are available to compute
the pointwise variance of the prior/posterior distribution and the trace
of the covariance operator.

The source code for `hIPPYlib` has been archived to Zenodo with the
linked DOI [10.5281/zenodo.596931](https://doi.org/10.5281/zenodo.596931).


# Acknowledgements

`hIPPYlib` development is partially supported by the U.S. National Science Foundation (NSF), Software Infrastructure for Sustained Innovation (SI2: SSE & SSI) Program under grants ACI-1550593, ACI-1550547; the Defense Advanced Research Projects Agency, Enabling Quantification of Uncertainty in Physical Systems Program under grant CBET-1508713; the US National Science Foundation, Division Of Chemical, Bioengineering, Environmental, & Transport Systems under grant CBET-1508713; and the Air Force Office of Scientific Research, Computational Mathematics program under grant FA9550- 12-1-81243.

# References
